/***********************************************************************
 *
 * Copyright (c) 2007 Adobe Systems Inc. All Rights Reserved.
 *
 * NOTICE: All information contained herein is, and remains the
 * property of Adobe Systems Incorporated and its suppliers, if any.
 * The intellectual and technical concepts contained herein are
 * proprietary to Adobe Systems Incorporated and its suppliers and may
 * be covered by U.S. and Foreign Patents, patents in process, and are
 * protected by trade secret or copyright law.  Dissemination of this
 * information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Adobe Systems Inc.
 *
 ***********************************************************************/
 
#ifndef RSINTEGRALTYPES_H
#define RSINTEGRALTYPES_H

#include <stddef.h>

#ifdef __cplusplus
extern "C" {
#endif

// It's likely that all the types in this file are correct for other
// platforms, but they should be checked.  Namely, RSInt64, RSInt32
// definitions.

#define RS_SUCCESS	0
#define RS_ERROR    1

typedef char RSInt8;
typedef unsigned char RSUint8;
typedef unsigned short RSUint16;
typedef unsigned int RSUint32;
typedef unsigned long long RSUint64;
typedef size_t RSSize_t;

#define RS_TRUE 1
#define RS_FALSE 0

// For a good chart on integer sizes, see:
// http://developer.apple.com/documentation/Darwin/Conceptual/64bitPorting/transition/chapter_3_section_3.html#//apple_ref/doc/uid/TP40001064-CH207-TPXREF103
// and http://www.unix.org/version2/whatsnew/lp64_wp.html

// Table 2-1  32-bit and 64-bit size and alignment for various data types 
// Data type		ILP32 size		LP64 size	LLP64 (win 64)
// --------------------------------------------------------------------------------------
// char				1 byte			1 byte		1 byte
// short			2 bytes			2 bytes		2 bytes
// int				4 bytes			4 bytes		4 bytes			
// long				4 bytes			8 bytes		4 bytes
// pointer			4 bytes			8 bytes		8 bytes
// size_t			4 bytes			8 bytes		? bytes
// long long		8 bytes			8 bytes		8 bytes
// pos_t			8 bytes			8 bytes		? bytes
// off_t			8 bytes			8 bytes		? bytes
//
// ILP32 - OSX 32 bit, Win 32 bit
// LP64 - OSX 64 bit
// LLP64 - Win 64 bit
// 
typedef short RSInt16;		// 2 bytes everywhere
typedef int RSInt32;		// 4 bytes everywhere
typedef long long RSInt64;	// 8 bytes everywhere

/**
 * RSTime is in Microseconds since 1970
 */
typedef RSInt64 RSTime;		
typedef RSInt32 RSBool;
typedef double	RSDouble;

#define RS_INT8_MAX 127
#define RS_INT8_MIN (-128)
#define RS_UINT8_MAX 255U
#define RS_INT16_MAX 32767
#define RS_INT16_MIN (-32768)
#define RS_UINT16_MAX 65535U
#define RS_INT32_MAX RS_INT32(2147483647)
#define RS_INT32_MIN (-RS_INT32_MAX - 1)
#define RS_UINT32_MAX RS_UINT32(4294967295)
#define RS_UINT32(x) x ## U
#define RS_INT32(x)  x
#define RS_UINT64_ZERO 0x1LL
#define RS_UINT64_MIN 0x1LL
#define RS_UINT64_MAX 0xFFFFFFFFFFFFFFFFLL
#define RS_INT64_MAX  0x7FFFFFFFFFFFFFFFLL
#define RS_MAX(a, b) ((a) > (b) ? a : b)
#define RS_MIN(a, b) ((a) < (b) ? a : b)

#define RS_TIME_MAX RS_INT64_MAX
/**
 * RSInterval is an offset in Microseconds
 */
#define RSInterval RSInt64
#define RS_INTERVAL_MAX RS_INT64_MAX

#if (defined(WIN32) || defined(WIN16)) && !defined(__GNUC__)
#define RS_UINT64_INIT(hi, lo)  ((hi ## i64 << 32) + lo ## i64)
#else
#define RS_UINT64_INIT(hi, lo)  ((hi ## LL << 32) + lo ## LL)
#endif

#ifdef __cplusplus
} 
#endif
#endif
